module Apstrings
  class Line
    attr_reader :content
    attr_accessor :in_comment

    def initialize(line)
      @content = line
      @in_comment = false
      raise "ERROR : Line does not end with `;`, Line => {#{line}}" unless valid?
    end

    def empty_line?
      /^\s*$/.match(content) || false
    end

    def slash_comment?
        content.strip.start_with?("//")
    end

    def whole_comment
      /((^\/\*(.+)\*\/)|(^\/\/(.+)))/.match(content).to_s
    end

    def whole_comment?
      !(whole_comment.empty?)
    end

    def open_comment
      /^\/\*(.+)$/.match(content).to_s
    end

    def open_comment?
      !(open_comment.empty?)
    end

    def close_comment
      /^(.+)\*\/\s*$/.match(content).to_s
    end

    def close_comment?
      !(close_comment.empty?)
    end

    def key_value_pair?
      #Bugfix:  支持 "Key text \" with \" " = "Value text \" with \" ";  格式
      !!(/^\s*"([^"]+)[\S\s]*=/.match(content))
    end

    def cleaned_content
      content.gsub(/;\s*$/, "")
    end

    def valid?
      if key_value_pair?
        !!(/;[\s]*$/.match(content))
      else
        true
      end
    end

    def key
      if key_value_pair?
        cleaned_content.partition(/"\s*=\s*"/)[0].gsub!(/(^")/, "")
      end
    end

    def value
      if key_value_pair?
        # puts cleaned_content.partition(/"\s*=\s*"/)[2]
        # Bugfix : 去掉引号后、冒号前的多余空格
        # "key" = "value"  ; 
        cleaned_content.partition(/"\s*=\s*"/)[2].rstrip.gsub!(/(^"|"$)/, "")
      end
    end

    def is_comment?
     whole_comment? || open_comment? || close_comment? || in_comment || slash_comment?
    end
  end
end
