/**
 *  Formats validate.
 *
 *  Example
 *   var master = 'Amount: %.2f  %2$lld  Object: %@ String: %s  Other: %u ';
 *   var slave = '数值: %.2f  %2$lld   OC对象: %@ 字符串: %s  其它: %u ';
 *   var slave_err = '数值: % f  %2$lld   OC对象: %@ 字符串: %s  其它: %u ';
 *   var result = formats_validate(master, slave) // true
 *   var result_err = formats_validate(master, slave_err) // false
 *
 *  This implements format validation in IEEE Std 1003.1, 2004 Edition .
 *  Ref:  http://pubs.opengroup.org/onlinepubs/009695399/functions/printf.html
 *
 *  Created by Jason Kaer on 6/17/16.
 */

/***
 * Add equals method to Array
 */
(function () {
  if (Array.prototype.equals)
    console.warn("Overriding existing Array.prototype.equals. Possible causes: New API defines the method, there's a framework conflict or you've got double inclusions in your code.");
// attach the .equals method to Array's prototype to call it on any array
  Array.prototype.equals = function (array) {
    // if the other array is a falsy value, return
    if (!array)
      return false;

    // compare lengths - can save a lot of time
    if (this.length != array.length)
      return false;

    for (var i = 0, l = this.length; i < l; i++) {
      // Check if we have nested arrays
      if (this[i] instanceof Array && array[i] instanceof Array) {
        // recurse into the nested arrays
        if (!this[i].equals(array[i]))
          return false;
      }
      else if (this[i] != array[i]) {
        // Warning - two different object instances will never be equal: {x:20} != {x:20}
        return false;
      }
    }
    return true;
  };
  // Hide method from for-in loops
  Object.defineProperty(Array.prototype, "equals", {enumerable: false});
}());


/***
 * Validate formats(i.e. %d %s ...) in slave according to master .
 * @param master The reference standard string in validate.
 * @param slave  The string to be validated.
 * @returns {boolean}
 */
function formats_validate(master, slave) {
  try {
    var format_m = formats_in_string(master);
    var format_s = formats_in_string(slave);
    return format_m.equals(format_s);
  } catch (err) {
    console.log(err);
    return false;
  }
}


/***
 * Find all formats in a string
 * @param input String to scan.
 * @returns {Array}
 */
function formats_in_string(input) {

  var fix_regex = /%\d\$/g ;
  var scan_regex = /%['\-\+\#0]*[shlqLztj\.\*\d]*[@%diDuUxXoOfeEgGcCsSpPaAFn]/g ;

  //Strip numbered format placeholders , e.g. %1$@ --> %@
  var fixed_input = input.replace(fix_regex, "%");
  var m;
  var result = [];
  while ((m = scan_regex.exec(fixed_input)) !== null) {
    if (m.index === scan_regex.lastIndex) {
      scan_regex.lastIndex++;
    }
    result.push(m[0])
  }

  return result;
}


